package com.paypal.hybris.addon.renderer;

import com.paypal.hybris.core.service.impl.DefaultPayPalConfigurationService;
import static com.paypal.hybris.core.constants.PaypalcoreConstants.PAYPAL_DISABLE_FUNDING_CART_PAGE;
import static com.paypal.hybris.core.constants.PaypalcoreConstants.PAYPAL_DISABLE_FUNDING_MARK_PAGE;
import static com.paypal.hybris.core.constants.PaypalcoreConstants.PAYPAL_DISABLE_FUNDING_MINI_CART;

import com.paypal.hybris.facade.facades.impl.DefaultPayPalButtonRenderDecisionMakerFacade;
import de.hybris.platform.addonsupport.renderer.impl.DefaultAddOnCMSComponentRenderer;
import de.hybris.platform.cms2.model.contents.components.AbstractCMSComponentModel;
import paypalhac.services.PayPalButtonConfigurationService;

import javax.annotation.Resource;
import javax.servlet.jsp.PageContext;
import java.util.Map;


public class PayPalDefaultAddOnCMSComponentRenderer<C extends AbstractCMSComponentModel>
    extends DefaultAddOnCMSComponentRenderer<C> {

    private static final String CLIENT_ID = "clientId";
    private static final String INTENT = "intent";
    private static final String COMMIT = "commit";
    private static final String PAYPAL_SDK_URL = "paypalSDKUrl";
    private static final String DISABLE_FUNDING = "disableFunding";
    private static final String PAYPAL_BUTTON_COMPONENT = "PayPalButtonComponent";
    private static final String PAYPAL_MINI_BUTTON_COMPONENT = "PayPalMiniButtonComponent";
    private static final String PAYPAL_MARK_COMPONENT = "PayPalMarkComponent";
    private static final String CUSTOM_JS_CONFIG = "customJsConfig";
    private static final String CART_BUTTON_CONFIG = "cart.button.config";
    private static final String MINI_CART_BUTTON_CONFIG = "mini.cart.button.config";
    private static final String MARK_BUTTON_CONFIG = "mark.button.config";
    private static final String PARTNER_ATTRIBUTION_ID = "partnerAttributionId";
    private static final String BUYER_COUNTRY = "buyerCountry";
    private static final String LOCALE = "locale";


    @Resource(name = "payPalButtonConfigurationService")
    private PayPalButtonConfigurationService payPalButtonConfigurationService;

    @Resource(name = "payPalConfigurationService")
    private DefaultPayPalConfigurationService defaultPayPalConfigurationService;

    @Resource(name = "payPalButtonRenderDecisionMakerFacade")
    private DefaultPayPalButtonRenderDecisionMakerFacade payPalButtonRenderDecisionMakerFacade;

    @Override
    protected Map<String, Object> exposeVariables(final PageContext pageContext,
        @SuppressWarnings("unused") final C component) //NOSONAR
    {
        final Map<String, Object> variables = getVariablesToExpose(pageContext, component);
        if (variables != null) {

            variables.forEach((key, value) -> pageContext
                .setAttribute(key, value, getScopeForVariableName(key)));

            pageContext.setAttribute(CLIENT_ID, defaultPayPalConfigurationService.getClientID(),
                getScopeForVariableName(CLIENT_ID));
            pageContext.setAttribute(INTENT, defaultPayPalConfigurationService.getPayPalIntent(),
                getScopeForVariableName(INTENT));
            pageContext.setAttribute(COMMIT, defaultPayPalConfigurationService.isCommitEnabled(),
                getScopeForVariableName(COMMIT));
            pageContext.setAttribute(PAYPAL_SDK_URL, defaultPayPalConfigurationService.getPayPalSdkJsUrl(),
                getScopeForVariableName(PAYPAL_SDK_URL));
            pageContext.setAttribute(PARTNER_ATTRIBUTION_ID, defaultPayPalConfigurationService.getPayPalPartnerAttributionId(),
                    getScopeForVariableName(PARTNER_ATTRIBUTION_ID));

            String customJsConfig;

            switch (component.getUid()) {
                case PAYPAL_BUTTON_COMPONENT:
                    customJsConfig = payPalButtonConfigurationService.getProperty(CART_BUTTON_CONFIG);
                    pageContext.setAttribute(DISABLE_FUNDING, payPalButtonRenderDecisionMakerFacade.getDisableFunding(PAYPAL_DISABLE_FUNDING_CART_PAGE),
                        getScopeForVariableName(DISABLE_FUNDING));
                    pageContext.setAttribute(BUYER_COUNTRY, defaultPayPalConfigurationService.getBuyerCountry(),
                        getScopeForVariableName(BUYER_COUNTRY));
                    break;
                case PAYPAL_MINI_BUTTON_COMPONENT:
                    customJsConfig = payPalButtonConfigurationService.getProperty(MINI_CART_BUTTON_CONFIG);
                    pageContext.setAttribute(DISABLE_FUNDING, payPalButtonRenderDecisionMakerFacade.getDisableFunding(PAYPAL_DISABLE_FUNDING_MINI_CART),
                        getScopeForVariableName(DISABLE_FUNDING));
                    pageContext.setAttribute(BUYER_COUNTRY, defaultPayPalConfigurationService.getBuyerCountry(),
                        getScopeForVariableName(BUYER_COUNTRY));
                    break;
                case PAYPAL_MARK_COMPONENT:
                    customJsConfig = payPalButtonConfigurationService.getProperty(MARK_BUTTON_CONFIG);
                    pageContext.setAttribute(DISABLE_FUNDING, payPalButtonRenderDecisionMakerFacade.getDisableFunding(PAYPAL_DISABLE_FUNDING_MARK_PAGE),
                        getScopeForVariableName(DISABLE_FUNDING));
                    //TODO variables for testing only, should be removed
                    pageContext.setAttribute(BUYER_COUNTRY, defaultPayPalConfigurationService.getBuyerCountry(),
                        getScopeForVariableName(BUYER_COUNTRY));
                    pageContext.setAttribute(LOCALE, defaultPayPalConfigurationService.getLPMLocale(),
                        getScopeForVariableName(LOCALE));
                    break;
                default:
                    customJsConfig = "";
            }

            if (customJsConfig.length() > 0 && !customJsConfig.endsWith(",")) {
                customJsConfig = "style: " + customJsConfig.replaceAll("[,{]", "$0 \n  ").replaceAll(":", "$0 ")
                    .replaceAll("}", " \n$0");
                customJsConfig += ",";
            }

            pageContext.setAttribute(CUSTOM_JS_CONFIG, customJsConfig, getScopeForVariableName(CUSTOM_JS_CONFIG));
        }
        return variables;
    }

}
