package com.paypal.hybris.b2baddon.controllers.misc;

import com.paypal.hybris.b2baddon.controllers.Paypalb2baddonControllerConstants;
import com.paypal.hybris.b2baddon.model.PayPalMiniCartComponentB2BModel;
import com.paypal.hybris.core.service.impl.DefaultPayPalConfigurationService;
import com.paypal.hybris.facade.facades.impl.DefaultPayPalButtonRenderDecisionMakerFacade;
import de.hybris.platform.acceleratorstorefrontcommons.controllers.AbstractController;
import de.hybris.platform.cms2.exceptions.CMSItemNotFoundException;
import de.hybris.platform.cms2.model.contents.components.SimpleCMSComponentModel;
import de.hybris.platform.cms2.servicelayer.services.CMSComponentService;
import de.hybris.platform.commercefacades.order.CartFacade;
import de.hybris.platform.commercefacades.order.data.CartData;
import org.apache.log4j.Logger;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;


@Controller
@Scope("tenant")
public class PayPalMiniCartController extends AbstractController {

    private static final String CART_DATA = "cartData";
    private static final String ENTRIES = "entries";
    private static final String NUMBER_ITEMS_IN_CART = "numberItemsInCart";
    private static final String NUMBER_SHOWING = "numberShowing";
    private static final String LIGHTBOX_BANNER_COMPONENT = "lightboxBannerComponent";
    private static final String EXPRESS_CHECKOUT_COMPONENT = "expressCheckoutComponent";
    private static final String EXPRESS_BILLING_AGREEMENT_FLOW = "isBillingAgreementFlow";

    private static final String PAYPAL_MINI_BUTTON_COMPONENT_B2B = "PayPalMiniButtonComponentB2B";
    private static final String PAYPAL_BILLING_AGREEMENT_EXPRESS_CHECKOUT = "PayPalBillingAgreementExpressCheckout";

    @Resource(name = "cartFacade")
    private CartFacade cartFacade;

    @Resource(name = "cmsComponentService")
    private CMSComponentService cmsComponentService;

    @Resource(name = "payPalButtonRenderDecisionMakerFacade")
    private DefaultPayPalButtonRenderDecisionMakerFacade decisionMakerFacade;

    @Resource(name = "payPalConfigurationService")
    private DefaultPayPalConfigurationService defaultPayPalConfigurationService;

    /**
     * Controller that handle minicart popup with embedded PayPal buttons
     *
     * @param componentUid
     * @param model
     * @return
     * @throws CMSItemNotFoundException
     */
    @RequestMapping(value = "/cart/rollover/paypal{componentUid}", method = RequestMethod.GET)
    public String rolloverMiniCartPopup(@PathVariable final String componentUid, final Model model)
        throws CMSItemNotFoundException {
        final CartData cartData = cartFacade.getSessionCart();
        model.addAttribute(CART_DATA, cartData);

        final PayPalMiniCartComponentB2BModel component = (PayPalMiniCartComponentB2BModel) cmsComponentService
            .getSimpleCMSComponent(componentUid);

        final List entries = cartData.getEntries();
        if (entries != null) {
            Collections.reverse(entries);
            model.addAttribute(ENTRIES, entries);

            model.addAttribute(NUMBER_ITEMS_IN_CART, Integer.valueOf(entries.size()));
            if (entries.size() < component.getShownProductCount()) {
                model.addAttribute(NUMBER_SHOWING, Integer.valueOf(entries.size()));
            } else {
                model.addAttribute(NUMBER_SHOWING, Integer.valueOf(component.getShownProductCount()));
            }
        }
        if (defaultPayPalConfigurationService.isExpressCheckoutEnabled()) {
            final SimpleCMSComponentModel payPalMiniButtonComponent = cmsComponentService
                .getSimpleCMSComponent(PAYPAL_MINI_BUTTON_COMPONENT_B2B);
            final SimpleCMSComponentModel payPalExpressCheckoutComponent = cmsComponentService
                .getSimpleCMSComponent(PAYPAL_BILLING_AGREEMENT_EXPRESS_CHECKOUT);
            model.addAttribute(LIGHTBOX_BANNER_COMPONENT, component.getLightboxBannerComponent());
            model.addAttribute(EXPRESS_CHECKOUT_COMPONENT,
                decisionMakerFacade.isExpressCheckoutStaticImageFlow(cartData) ?
                    payPalExpressCheckoutComponent : payPalMiniButtonComponent);
            model.addAttribute(EXPRESS_BILLING_AGREEMENT_FLOW,
                decisionMakerFacade.isExpressCheckoutBillingAgreementFlow());
        }
        return Paypalb2baddonControllerConstants.Views.Fragments.Cart.CART_POPUP;
    }
}
