package com.paypal.hybris.core.commands.impl;

import com.paypal.hybris.core.commands.PayPalAbstractCommand;

import com.paypal.hybris.data.PayPalOrderRequestData;
import com.paypal.orders.AmountWithBreakdown;
import com.paypal.orders.OrdersPatchRequest;
import com.paypal.orders.Patch;
import de.hybris.platform.payment.AdapterException;
import de.hybris.platform.payment.commands.Command;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;


public class DefaultPayPalUpdateOrderAmountCommand extends PayPalAbstractCommand
    implements Command<PayPalOrderRequestData, Boolean> {

    private static final String PATH = "/purchase_units/@reference_id=='default'/amount";
    private static final String REPLACE = "replace";

    private static final Logger LOG = Logger.getLogger(DefaultPayPalUpdateOrderAmountCommand.class);

    @Override
    public Boolean perform(final PayPalOrderRequestData orderDetails) {
        final OrdersPatchRequest ordersPatchRequest = new OrdersPatchRequest(orderDetails.getOrderId());
        List<Patch> patches = new ArrayList<>();
        patches.add(new Patch().op(REPLACE).path(PATH)
            .value(new AmountWithBreakdown().currencyCode(orderDetails.getCurrency()).value(orderDetails.getAmount())));
        ordersPatchRequest.requestBody(patches);
        try {
            createClient().execute(ordersPatchRequest);
        } catch (IOException e) {
            LOG.error("Updating Order amount failed: ", e);
            throw new AdapterException(getDescriptionFromPayPalErrorMessage(e.getMessage()));
        }
        return true;
    }
}
