package com.paypal.hybris.core.service.impl;

import com.paypal.hybris.core.model.PayPalCreditCardPaymentInfoModel;
import com.paypal.hybris.core.service.PayPalCustomerAccountService;
import de.hybris.platform.commercefacades.order.data.CCPaymentInfoData;
import de.hybris.platform.commerceservices.customer.impl.DefaultCustomerAccountService;
import de.hybris.platform.core.enums.CreditCardType;
import de.hybris.platform.core.model.order.payment.CreditCardPaymentInfoModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.core.model.user.CustomerModel;

import java.util.UUID;


public class DefaultPayPalCustomerAccountService extends DefaultCustomerAccountService implements
    PayPalCustomerAccountService {

    private static final int ZERO = 0;

    private DefaultPayPalConfigurationService defaultPayPalConfigurationService;

    @Override
    public PayPalCreditCardPaymentInfoModel createPayPalPaymentSubscription(final CustomerModel customerModel,
        final CCPaymentInfoData ccPaymentInfoData, final AddressModel billingAddress) {
        final PayPalCreditCardPaymentInfoModel payPalPaymentInfoModel = getModelService()
            .create(PayPalCreditCardPaymentInfoModel.class);

        populateDumbCCInfo(payPalPaymentInfoModel, customerModel);
        populatePayPalPaymentInfo(payPalPaymentInfoModel, ccPaymentInfoData);

        payPalPaymentInfoModel.setBillingAddress(billingAddress);
        payPalPaymentInfoModel.setIntent(defaultPayPalConfigurationService.getPayPalIntent());
        if (billingAddress.getOwner() == null) {
            billingAddress.setOwner(payPalPaymentInfoModel);
        }
        getModelService().saveAll(billingAddress, payPalPaymentInfoModel);
        getModelService().refresh(customerModel);

        addPaymentInfo(customerModel, payPalPaymentInfoModel);
        if (ccPaymentInfoData.isDefaultPaymentInfo()) {
            customerModel.setDefaultPaymentInfo(payPalPaymentInfoModel);
            getModelService().save(customerModel);
        }
        return payPalPaymentInfoModel;
    }

    @Override
    public PayPalCreditCardPaymentInfoModel updatePayPalPaymentSubscription(final CustomerModel customerModel,
        final CCPaymentInfoData ccPaymentInfoData, final AddressModel billingAddress,
        PayPalCreditCardPaymentInfoModel paymentInfo) {
        populatePayPalPaymentInfo(paymentInfo, ccPaymentInfoData);

        paymentInfo.setBillingAddress(billingAddress);
        if (billingAddress.getOwner() == null) {
            billingAddress.setOwner(paymentInfo);
        }
        getModelService().saveAll(billingAddress, paymentInfo);
        getModelService().refresh(customerModel);

        addPaymentInfo(customerModel, paymentInfo);
        return paymentInfo;
    }

    private void populatePayPalPaymentInfo(final PayPalCreditCardPaymentInfoModel payPalPaymentInfoModel,
        final CCPaymentInfoData ccPaymentInfoData) {
        payPalPaymentInfoModel.setPayerEmail(ccPaymentInfoData.getPayerEmail());
        payPalPaymentInfoModel.setPayPalOrderId(ccPaymentInfoData.getPayPalOrderId());
        payPalPaymentInfoModel.setPayerId(ccPaymentInfoData.getPayerId());
        payPalPaymentInfoModel.setBillingAgreementId(ccPaymentInfoData.getBillingAgreementId());
        payPalPaymentInfoModel.setSaved(ccPaymentInfoData.isSaved());
        payPalPaymentInfoModel.setSubscriptionId(ccPaymentInfoData.getSubscriptionId());
        payPalPaymentInfoModel.setCcOwner(ccPaymentInfoData.getAccountHolderName());
        payPalPaymentInfoModel.setType(CreditCardType.valueOf(ccPaymentInfoData.getCardType().toUpperCase()));
    }

    private void populateDumbCCInfo(final CreditCardPaymentInfoModel creditCardPaymentInfoModel,
        final CustomerModel customerModel) {
        creditCardPaymentInfoModel.setCode(customerModel.getUid() + "_" + UUID.randomUUID());
        creditCardPaymentInfoModel.setUser(customerModel);
        creditCardPaymentInfoModel.setValidToMonth(String.valueOf(ZERO));
        creditCardPaymentInfoModel.setValidToYear(String.valueOf(ZERO));
        creditCardPaymentInfoModel.setNumber(getMaskedCardNumber("****************"));
    }

    public DefaultPayPalConfigurationService getDefaultPayPalConfigurationService() {
        return defaultPayPalConfigurationService;
    }

    public void setDefaultPayPalConfigurationService(DefaultPayPalConfigurationService defaultPayPalConfigurationService) {
        this.defaultPayPalConfigurationService = defaultPayPalConfigurationService;
    }
}
