/**
 *
 */
package com.paypal.hybris.core.service.impl;

import static com.paypal.hybris.core.constants.PaypalcoreConstants.PAYPAL_ACCEPTED_PAYMENT_METHODS;
import static com.paypal.hybris.core.constants.PaypalcoreConstants.PAYPAL_BILLING_AGREEMENT_ENABLED;
import static com.paypal.hybris.core.constants.PaypalcoreConstants.PAYPAL_EXPRESS_CHECKOUT_ENABLED;
import static com.paypal.hybris.core.constants.PaypalcoreConstants.PAYPAL_IMAGES_PREFIX;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import de.hybris.bootstrap.annotations.UnitTest;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import java.util.Map;
import org.apache.commons.configuration.Configuration;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

@UnitTest
public class DefaultPayPalConfigurationServiceTest {

    private static final String VISA_METHOD = "visa";
    private static final String MASTERCARD_METHOD = "mastercard";
    private static final String VISA_IMAGE = "visaImage";
    private static final String MASTERCARD_IMAGE = "mastercardImage";
    private Configuration configuration;

    @Mock
    private ConfigurationService configurationService;

    @InjectMocks
    private DefaultPayPalConfigurationService unit;

    @Before
    public void setUp() {
        MockitoAnnotations.initMocks(this);
        configuration = Mockito.mock(Configuration.class);
        when(configurationService.getConfiguration()).thenReturn(configuration);

        when(configuration.getBoolean(PAYPAL_EXPRESS_CHECKOUT_ENABLED)).thenReturn(true);
        when(configuration.getBoolean(PAYPAL_BILLING_AGREEMENT_ENABLED)).thenReturn(false);
        when(configuration.getString(PAYPAL_ACCEPTED_PAYMENT_METHODS))
            .thenReturn(VISA_METHOD + ";" + MASTERCARD_METHOD);

        when(configuration.getString(PAYPAL_IMAGES_PREFIX + VISA_METHOD)).thenReturn(VISA_IMAGE);
        when(configuration.getString(PAYPAL_IMAGES_PREFIX + MASTERCARD_METHOD)).thenReturn(MASTERCARD_IMAGE);
    }

    @Test
    public void testIsExpressCheckoutEnabled() {
        final boolean aBoolean = configurationService.getConfiguration().getBoolean(PAYPAL_EXPRESS_CHECKOUT_ENABLED);
        Assert.assertTrue(aBoolean);
        verify(configuration).getBoolean(PAYPAL_EXPRESS_CHECKOUT_ENABLED);
    }

    @Test
    public void testIsBillingAgreementEnabled() {
        final boolean b = configurationService.getConfiguration().getBoolean(PAYPAL_BILLING_AGREEMENT_ENABLED);
        Assert.assertFalse(b);
        verify(configuration).getBoolean(PAYPAL_BILLING_AGREEMENT_ENABLED);
    }

    @Test
    public void testGetAcceptedPaymentMethodImages() {
        final Map<String, String> result = unit.getAcceptedPaymentMethodImages();
        Assert.assertEquals(VISA_IMAGE, result.get(VISA_METHOD));
        Assert.assertEquals(MASTERCARD_IMAGE, result.get(MASTERCARD_METHOD));
        verify(configuration).getString(PAYPAL_ACCEPTED_PAYMENT_METHODS);
    }
}
