package com.paypal.hybris.core.service.impl;

import com.paypal.hybris.core.commands.impl.DefaultPayPalCreateBillingAgreementCommand;
import com.paypal.hybris.core.commands.impl.DefaultPayPalCreateBillingAgreementTokenCommand;
import com.paypal.hybris.core.commands.impl.DefaultPayPalGetOrderDetailsCommand;
import com.paypal.hybris.core.commands.impl.DefaultPayPalUpdateOrderAmountCommand;
import com.paypal.hybris.core.service.PayPalPaymentService;
import com.paypal.hybris.data.PayPalOrderRequestData;
import de.hybris.platform.payment.commands.factory.CommandFactory;
import de.hybris.platform.payment.commands.factory.CommandFactoryRegistry;
import de.hybris.platform.payment.commands.factory.CommandNotSupportedException;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import java.util.UUID;

import static com.paypal.hybris.core.constants.PaypalcoreConstants.PAYPAL_PROVIDER_NAME;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;


public class DefaultPayPalPaymentServiceTest {

    @Mock
    private CommandFactoryRegistry commandFactoryRegistry;

    @InjectMocks
    private PayPalPaymentService unit = new DefaultPayPalPaymentService();

    @Before
    public void setUp() {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void shouldGetOrderDetails() throws CommandNotSupportedException {
        final String expectedRequest = UUID.randomUUID().toString();
        final CommandFactory commandFactory = mock(CommandFactory.class);
        DefaultPayPalGetOrderDetailsCommand command = mock(DefaultPayPalGetOrderDetailsCommand.class);
        when(commandFactoryRegistry.getFactory(PAYPAL_PROVIDER_NAME)).thenReturn(commandFactory);
        when(commandFactory.createCommand(DefaultPayPalGetOrderDetailsCommand.class)).thenReturn(command);

        unit.getOrderDetails(expectedRequest);

        verify(command).perform(expectedRequest);
    }

    @Test
    public void shouldCreateBillingAgreement() throws CommandNotSupportedException {
        final String expectedRequest = UUID.randomUUID().toString();
        final CommandFactory commandFactory = mock(CommandFactory.class);
        DefaultPayPalCreateBillingAgreementCommand command = mock(DefaultPayPalCreateBillingAgreementCommand.class);
        when(commandFactoryRegistry.getFactory(PAYPAL_PROVIDER_NAME)).thenReturn(commandFactory);
        when(commandFactory.createCommand(DefaultPayPalCreateBillingAgreementCommand.class)).thenReturn(command);

        unit.createBillingAgreement(expectedRequest);

        verify(command).perform(expectedRequest);
    }

    @Test
    public void shouldInvokeCommandUpdateOrderAmountDetails() throws CommandNotSupportedException {
        final PayPalOrderRequestData requestData = mock(PayPalOrderRequestData.class);
        when(requestData.getOrderId()).thenReturn(UUID.randomUUID().toString());
        when(requestData.getCurrency()).thenReturn(UUID.randomUUID().toString());
        when(requestData.getAmount()).thenReturn(UUID.randomUUID().toString());
        final CommandFactory commandFactory = mock(CommandFactory.class);
        DefaultPayPalUpdateOrderAmountCommand command = mock(DefaultPayPalUpdateOrderAmountCommand.class);
        when(commandFactoryRegistry.getFactory(PAYPAL_PROVIDER_NAME)).thenReturn(commandFactory);
        when(commandFactory.createCommand(DefaultPayPalUpdateOrderAmountCommand.class)).thenReturn(command);

        unit.updateOrderAmountDetails(requestData.getOrderId(), requestData.getCurrency(), requestData.getAmount());

        verify(command).perform(any());
    }

    @Test
    public void shouldInvokeCommandCreateBillingAgreementToken() throws CommandNotSupportedException {
        final CommandFactory commandFactory = mock(CommandFactory.class);
        DefaultPayPalCreateBillingAgreementTokenCommand command = mock(DefaultPayPalCreateBillingAgreementTokenCommand.class);
        when(commandFactoryRegistry.getFactory(PAYPAL_PROVIDER_NAME)).thenReturn(commandFactory);
        when(commandFactory.createCommand(DefaultPayPalCreateBillingAgreementTokenCommand.class)).thenReturn(command);

        unit.createBillingAgreementToken(true);

        verify(command).perform(true);
    }
}
