package com.paypal.hybris.facade.facades.impl;

import com.paypal.hybris.data.PayPalAddressDetailsData;
import com.paypal.hybris.facade.facades.PayPalCheckoutFacade;
import com.paypal.hybris.facade.facades.PayPalGuestCheckoutFacade;
import de.hybris.platform.commercefacades.customer.CustomerFacade;
import de.hybris.platform.commerceservices.customer.DuplicateUidException;
import de.hybris.platform.servicelayer.session.SessionService;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

public class DefaultPayPalGuestCheckoutFacade implements PayPalGuestCheckoutFacade {

    private static final String ANONYMOUS_USER_UID = "anonymous";
    private static final String ANONYMOUS_CHECKOUT = "anonymous_checkout";
    private static final String ANONYMOUS_CHECKOUT_GUID = "anonymous_checkout_guid";
    private static final Logger LOG = Logger.getLogger(DefaultPayPalGuestCheckoutFacade.class);

    private CustomerFacade customerFacade;
    private PayPalCheckoutFacade checkoutFacade;
    private SessionService sessionService;

    @Override
    public void processCheckoutForAnonymous(PayPalAddressDetailsData addressDetailsData) {
        try {
            if (ANONYMOUS_USER_UID.equals(checkoutFacade.getCurrentSessionUserUid())) {
                customerFacade.createGuestUserForAnonymousCheckout(addressDetailsData.getEmail(),
                    addressDetailsData.getFirstName());
            } else if (!addressDetailsData.getEmail()
                .equalsIgnoreCase(StringUtils.substringAfter(checkoutFacade.getCurrentSessionUserUid(), "|"))) {
                customerFacade.createGuestUserForAnonymousCheckout(addressDetailsData.getEmail(),
                    addressDetailsData.getFirstName());
                sessionService.setAttribute(ANONYMOUS_CHECKOUT_GUID,
                    StringUtils.substringBefore(checkoutFacade.getCurrentSessionUserUid(), "|"));
            }
            sessionService.setAttribute(ANONYMOUS_CHECKOUT, true);
        } catch (final DuplicateUidException e) {
            LOG.warn("Guest registration failed, PayPal email: " + addressDetailsData.getEmail() + ".\n details" + e);
        }
    }

    public CustomerFacade getCustomerFacade() {
        return customerFacade;
    }

    public void setCustomerFacade(CustomerFacade customerFacade) {
        this.customerFacade = customerFacade;
    }

    public SessionService getSessionService() {
        return sessionService;
    }

    public void setSessionService(SessionService sessionService) {
        this.sessionService = sessionService;
    }

    public PayPalCheckoutFacade getCheckoutFacade() {
        return checkoutFacade;
    }

    public void setCheckoutFacade(PayPalCheckoutFacade checkoutFacade) {
        this.checkoutFacade = checkoutFacade;
    }

}