import { Injectable } from '@angular/core';
import {PaypalUtilsService} from '../utils';
import {PageType, PayPalButtonStyle, PaypalConfiguration, PayPalPaymentRequest} from '../../models';
import { PaypalPaymentDetailsService } from '../../facade/paypal-payment-details.service';
import {PaypalProcessPaymentCheckoutService} from './paypal-process-payment-checkout.service';

@Injectable({
  providedIn: 'root'
})
export class PaypalMarkCheckoutService {

  constructor(
    protected paypalUtilsService: PaypalUtilsService,
    protected paypalPaymentDetailsService: PaypalPaymentDetailsService,
    protected paypalProcessPaymentCheckoutService: PaypalProcessPaymentCheckoutService
  ) { }

  initializePayPalCheckout(configurationData: PaypalConfiguration, pageType: PageType, payPalButtonContainer, buttonStyle: PayPalButtonStyle): void{

    if (configurationData.isBillingAgreementFlowEnabled){
      this.paypalUtilsService.loadPayPalSDK(configurationData, pageType, () => {
        this.renderBillingAgreementButton(payPalButtonContainer, buttonStyle, pageType);
      });
    }else {
      this.paypalUtilsService.loadPayPalSDK(configurationData, pageType, () => {
        this.renderMarkButton(payPalButtonContainer, buttonStyle, pageType);
      });
    }

  }

  renderBillingAgreementButton(payPalButtonContainer, buttonStyle: PayPalButtonStyle, pageType: PageType): void{
    try {
    if (typeof (window as any).paypal !== undefined) {
      (window as any).paypal.Buttons({
        style: buttonStyle,

          createBillingAgreement: () => {
          return this.paypalPaymentDetailsService.createBillingAgreementToken().toPromise()
              .then((res) => {
              return res.tokenId;
            });
          },
          onApprove: (data) => {

            return this.paypalProcessPaymentCheckoutService.processBillingAgreementPaypalResponse(data, pageType);
          },
          onError: (err) => {
            console.error('Error: ' + err, err);
          }
        }).render(payPalButtonContainer);
    }
    } catch (err) {
      console.log(err.message);
    }
  }

  renderMarkButton(payPalButtonContainer, buttonStyle: PayPalButtonStyle, pageType: PageType): void{
    let fundingSource;
    try {
    if (typeof (window as any).paypal !== undefined) {
      (window as any).paypal.Buttons(
        {
          style: buttonStyle,

          onClick: (button) => {
            fundingSource = button.fundingSource;
          },

          createOrder: () => {
            return this.paypalPaymentDetailsService.createOrder(fundingSource)
              .toPromise().then((res) => {
                return res.orderId;
              });
          },
          onApprove: (data) => {

            return this.paypalProcessPaymentCheckoutService.processPaypalResponse(data, pageType);

          },
          onError: (err) => {
            console.error('Error: ' + err, err);
          }
        }).render(payPalButtonContainer);
    }
    } catch (err) {
      console.log(err.message);
    }
  }
}
