import {Injectable} from '@angular/core';
import {PaypalPaymentDetailsService} from '../../facade/paypal-payment-details.service';
import {Fields, PageType, PayPalBillingAgreementPaymentRequest, PayPalPaymentRequest} from '../../models';
import {takeWhile} from 'rxjs/operators';
import {ActivatedRoute, Router} from '@angular/router';
import {PaypalUtilsService} from '../utils/paypal-utils.service';
import {Location} from '@angular/common';
import {ActiveCartService, CheckoutService, MultiCartService, RoutingService} from '@spartacus/core';
import {CheckoutDetailsService} from '@spartacus/storefront';
import {combineLatest} from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class PaypalProcessPaymentCheckoutService {

  constructor(
    protected paypalPaymentDetailsService: PaypalPaymentDetailsService,
    protected paypalUtilsService: PaypalUtilsService,
    protected router: Router,
    protected checkoutService: CheckoutService,
    protected activeCartService: ActiveCartService,
    protected multiCartService: MultiCartService
  ) { }

  public processPaypalResponse(data, pageType: PageType, paymentMethodType?: string): void {
    let cartId;
    this.activeCartService
    .getActiveCartId()
    .subscribe((activeCartId) => (cartId = activeCartId))
    .unsubscribe();
    let payPalPaymentRequest: PayPalPaymentRequest;

    if (pageType === PageType.MARK) {
      payPalPaymentRequest = {
        orderId: data.orderID,
        payPalFlow: 'MARK_CHECKOUT'
      };
    } else if (pageType === PageType.CART)
      payPalPaymentRequest = {
        orderId: data.orderID,
        payPalFlow: 'EXPRESS_CHECKOUT'
      };

    this.paypalPaymentDetailsService.createPayPalPaymentMethod(payPalPaymentRequest, Fields.Full, paymentMethodType)
      .pipe(takeWhile(paymentDetails => paymentDetails === undefined || paymentDetails.id === undefined, true))
      .subscribe(res => {
        this.multiCartService.reloadCart(cartId);
        this.checkoutService.loadCheckoutDetails(cartId);
        if (res.cardType.code === 'local_payment'){
          this.checkoutService.placeOrder(true);
        }else {
          this.router.navigate(['/checkout/review-order']);
        }
      });
  }

  public processBillingAgreementPaypalResponse(data, pageType: PageType): void {
    let cartId;
    this.activeCartService
    .getActiveCartId()
    .subscribe((activeCartId) => (cartId = activeCartId))
    .unsubscribe();

    let payPalBillingAgreementPaymentRequest: PayPalBillingAgreementPaymentRequest;

    if (pageType === PageType.MARK) {
      payPalBillingAgreementPaymentRequest = {
        billingAgreementTokenId: data.billingToken,
        flow: 'BILLING_AGREEMENT_FLOW',
        isSavePaymentInfo: this.paypalUtilsService.savePaymentInfo
      };
    } else if (pageType === PageType.CART) {
      payPalBillingAgreementPaymentRequest = {
        billingAgreementTokenId: data.billingToken,
        flow: 'EXPRESS_BILLING_AGREEMENT_FLOW',
        isSavePaymentInfo: true
      };
    }

    this.paypalPaymentDetailsService.createPayPalBillingAgreementPayment(payPalBillingAgreementPaymentRequest, Fields.Full)
      .pipe(takeWhile(paymentDetails => paymentDetails === undefined || paymentDetails.id === undefined, true))
      .subscribe(res => {
        this.multiCartService.reloadCart(cartId);
        this.checkoutService.loadCheckoutDetails(cartId);
        this.router.navigate(['/checkout/review-order']);
      });
  }
}
