import {Injectable} from '@angular/core';
import {HttpErrorResponse, HttpRequest} from '@angular/common/http';
import {BadRequestHandler, GlobalMessageService, GlobalMessageType, HttpResponseStatus, Priority} from '@spartacus/core';

@Injectable({
  providedIn: 'root'
})
export class PaypalErrorHandlerService extends BadRequestHandler {
  responseStatus = HttpResponseStatus.BAD_REQUEST;

  constructor(
    protected globalMessageService: GlobalMessageService,
  ){
    super(globalMessageService);
  }

  handleError(
    request: HttpRequest<any>,
    errorResponse: HttpErrorResponse
  ): void {
    super.handleError(request, errorResponse);
    if (errorResponse.error?.errors?.length > 0) {
      errorResponse.error?.errors.forEach((error) => {
        if (error.type === 'AdapterError') {
          this.globalMessageService.add(
            { key: 'error.adapterError' },
            GlobalMessageType.MSG_TYPE_ERROR
          );
        }
      });
    }
  }

  getPriority(): Priority {
    return Priority.NORMAL;
  }
}
